'''
base 0.4
Basic scanners for sagator.

(c) 2003 Jan ONDREJ (SAL) <ondrejj@salstar.sk>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
    
'''

from avlib import *
import re,string

class regexp_scan(ascanner):
  '''
  Primitive regexp pattern scanner.
  
  Usage:
    regexp_scan({'VirName':['Regular_Expression_Pattern...',...],...})
  '''
  name='RegExpScanner()'
  def __init__(self,regexps):
      self.regexps=regexps
  def scanstream(self,stream):
      for vname,regs in self.regexps.items():
        for reg in regs:
          if re.search(reg,stream):
            if reg==regs[-1]:
              return vname,['regexp: '+vname+' FOUND, '+reg+'\n']
          else:
            break
      return '',[]

class string_scan(ascanner):
  '''
  Primitive string pattern scanner.
  
  Usage:
    string_scan({'VirName':['Pattern...',...],...})
  '''
  name='StringScanner()'
  def __init__(self,strings):
      self.strings=strings
  def scanstream(self,stream):
      for vname,regs in self.strings.items():
        for reg in regs:
          if string.find(stream,reg)>=0:
            if reg==regs[-1]:
              return vname,['string: '+vname+' FOUND, '+reg+'\n']
          else:
            break
      return '',[]

class stream2file(ascanner):
  '''
  stream2file interscanner
  
  You can use it to save streams into files.
  '''
  name='stream2file()'
  is_multistream=1
  def __init__(self,scanner,addheader=0):
      self.name=self.name[:-1]+scanner.name+')'
      self.scanner=scanner
      self.addheader=addheader
      self.filename=''
      self.mboxdir=''
      self.filenames=[]
      self.counter=1
  def addstream(self,stream):
      if stream:
        if self.mboxdir=='':
          self.mboxdir=smktemp('.mbd')
          os.mkdir(self.mboxdir,0770)
        if self.filename=='':
          self.filename='part-%06d' % (self.counter)
          self.counter=self.counter+1
        mboxfile=self.mboxdir+'/'+self.filename
        creatembox(mboxfile,stream,self.addheader)
        self.filenames.append(self.filename)
  def delstream(self):
      for fn in self.filenames:
        os.unlink(self.mboxdir+'/'+fn)
      os.rmdir(self.mboxdir)
  def scanstream(self,stream):
      vir,ret='',[]
      self.addstream(stream)
      try:
        vir,ret=self.scanner.scanfile(self.filenames,self.mboxdir)
      except:
        self.delstream()
        raise
      self.delstream()
      return vir,ret
  def scanmultistream(self,streams):
      vir,ret='',[]
      if type(streams)==type({}):
        for filename,stream in streams.items():
          self.addstream(stream,filename)
      else:
        for stream in streams:
          self.addstream(stream)
      try:
        vir,ret=self.scanner.scanfile(self.filenames,self.mboxdir)
      except:
        self.delstream()
        raise
      self.delstream()
      return vir,ret

class l_return(ascanner):
  name='l_return()'
  def __init__(self,vir='',ret=[]):
      self.vir=''+vir
      self.ret=[]+ret
  def scanstream(self,stream):
      return self.vir,self.ret
  def scanfile(self,files,dirname=''):
      return self.vir,self.ret

class match_all(ascanner):
  name='match_all()'
  def __init__(self,*scanners):
      self.scanners=scanners
      n=[]
      for s in scanners:
        n=n+[s.name]
      self.name=self.name[:-1]+string.join(n,',')+')'
  def scanstream(self,stream):
      vir,ret='',[]
      for s in self.scanners:
        v,r=s.scanstream(stream)
        if v:
          if vir:
            vir=vir+','+v
          else:
            vir=v
          ret=ret+r
        else:
          return '',[]
      return vir,ret

class match_any(match_all):
  name='match_any()'
  def scanstream(self,stream):
      vir,ret='',[]
      for s in self.scanners:
        try:
          vir,ret=s.scanstream(stream)
          return vir,ret
        except:
          debug_traceback(4,'')
          debug(3,self.name,": scanner ",s.name," failed: ",
                traceback_value_str())
      debug(3,self.name,": All scanners failed")
      raise ScannerError,''
